#!/bin/bash 
# This script can be used with SafeSquid's External Parser, to create a browser based file server
# To use this script, create a profile, say - "INTRANET", for a non-existant hostname, say "dummy"
# Now, in the External parser, set the fullpath of this file as the 'executable' and also specify the 
# full-path of the parent folder that contains the files and folder that you wish to share, as the first argument.
# For example, if this script-file shall be stored as /opt/safesquid/scripts/webserver.sh & you wish to share a folder called 
# /home/shared_files, then you should set 'executable' as:
# 		sh /opt/safesquid/scripts/webserver.sh /home/shared_files
# set 'profiles' to "INTRANET" ; set 'Type' to "pipe" ; Select the check box for 'Applies to Requests'
#	and enable the option for 'Run Once per session' to "NO" ; leave all other options unselected
# Make sure that your instance of SafeSquid has proper file permissions to execute opt/safesquid/scripts/webserver.sh and 
# read /home/shared_files folder.
# Now whenever people who use your proxy services try to access http://dummy/ they will be able to access the contents in 
# /home/shared_files
# You can try hacking this script a bit more for other possibilities!

USER="${USERNAME}"
CUR_TIME=`date +%c`
WEB=( `set | grep -i "CLIENT\[host\]" | awk 'BEGIN {FS="="} {print $2}'` )

[ x"${HTTP_HOST}" == "x" ] && HTTP_HOST="${WEB[0]}"

# If no folder is specified in the request URL, set the file-server root to the first argument supplied to the executable.
	# [ x"${HTTP_FILE}" == x"/" ] && BASE_DIR=$1 || BASE_DIR=${HTTP_FILE}
	# [ x"${HTTP_FILE}" == x"/" ] && BASE_DIR=$1

	ROOT_DIR=$1
	TARGET=${ROOT_DIR}${HTTP_FILE}
# Also take some security precautions so that requests folders outside the file-server root
# are returned to the file-server root

	# Calculate the length of the root-directory variable $1
#		DIR_ROOT_LENGTH=${#1}

	# For security, truncate the characters from the beginning of the HTTP_FILE and prefix the root-dir
#		BASE_DIR=$1${HTTP_FILE:${DIR_ROOT_LENGTH}}

# Set Default Mime type of all output content to text/html
	MIME="text/html"

# Find out if the request is for a file or a directory
	REQUEST="ERROR"
	[ -f "${TARGET}" ] && REQUEST="FILE"
	[ -d "${TARGET}" ] && REQUEST="DIRECTORY"

#	[ "${HTTP_FILE}" == "${HTTP_FILE%/}" ] && REQUEST="FILE" || REQUEST="DIRECTORY"
	
# If the REQUEST is for a FILE, Determine the extension of the file
		if [ x"${REQUEST}" == x"FILE" ] ; then 
			EXT=`expr "${HTTP_FILE}" : '.*\(\.\(.*\)\)'`
			# If the EXTension is for php then set REQUESTed FILE must be EXEC-uted
			[ x"${EXT}" == "x.php" ] && REQUEST="EXEC"
		fi
	
	
# Set APPROPRIATE ACTION
	case ${REQUEST} in
		FILE) ACTION="DOWNLOAD"
		;;
		DIRECTORY) ACTION="BROWSE_DIR"
		;;
		EXEC) ACTION="EXECUTE"
		;;
		ERROR) ACTION="SHOW_ERROR"
		;;
		*)
		;;
	esac

	
#	[ -f ${BASE_DIR} ] && DOWNLOAD="Y"

HEADERS ()
	{
	# Set up the HTTP Headers
		echo "HTTP/1.1 200 OK"
		echo "Date: ${CUR_TIME}"
		echo "Server: SafeSquid/${VERSION}"
		echo "Connection: Close"
		echo "Content-Type: ${MIME}"
		[ x"${CONTENT_LENGTH}" != "x" ] && echo "Content-Length: ${CONTENT_LENGTH[0]}"
		
		echo ""
	return;
	}
 
TOP ()
	{
	# Create the proper leading section for an HTML page	
		echo "<HTML>"
		echo "<HEAD>"
#		echo "<TITLE>" "Directory Listing of ${BASE_DIR} at ${CUR_TIME}" "</TITLE>"
	#echo "<tr><th width=\"100%\" colspan=\"4\">SafeSquid based web-fileserver runing on ${WEB[0]}</th></tr>"
		echo "<TITLE>" "${USER} accessing SafeSquid based web-fileserver runing on ${WEB[0]} at ${CUR_TIME}" "</TITLE>"
		echo "<base href=\"""http://${WEB[0]}${HTTP_FILE}\" />"
#		echo "<base href=\"""${HTTP_FILE}\" />"
		echo "<link rel=\"stylesheet\" type=\"text/css\" href=\"http://${INTERFACE}:${PORT}/safesquid.cfg/template/interface.css\" />"
		echo "</HEAD>"
		echo "<BODY>"
#		set
#		echo "${CLIENTID}"
		echo "<table class=\"header\" width=\"100%\">"
		echo "<tr ><td valign=\"middle\" align=\"left\" width=\"50%\" ><a href=\"http://"${WEB[0]}"\"> <img src=\"http://${INTERFACE}:${PORT}/safesquid.cfg/template/logo\" border=\"0\" width=\"195\" height=\"42\"></a> </td></tr>"
		echo "</table>"

	return;
	}

BOTTOM ()
	{
	# Create the proper closing section for an HTML page
#		set
		echo "</BODY>""</HTML>"
	return;
	}

SHOW_DIR ()
	{
	# Parse the folder and create a tabulated display
		echo "<table width=\"100%\" cellpadding=\"2\">"
		echo "<tr class=\"list\"><td ><a href=\"http://"${WEB[0]}"\">  HOME </a> </td><td colspan=\"2\" align=\"right\" ><a href=\"${HTTP_FILE}..\" > <img src=\"http://${INTERFACE}:${PORT}/safesquid.cfg/template/uparrow\" border=\"0\" height=\"27\" width=\"27\"> One UP </a> </td></tr>"
		echo "<tr class=\"listhead\"><td> Directory Listing for ${HTTP_FILE} </td><td> Size </td><td> Date </td></tr> "
		echo "${CONTENT}" | awk '{print "<tr class=\"listeven\"><td width=\"50%\">" "<a href=\"" "'"${HTTP_FILE}"'" $7 "\">" $7 "</a>" "</td><td width=\"25%\">" $3 "</td><td nowrap width=\"25%\" colspan=\"2\">" $4 , $5"</td></tr>"}'
		echo "</table>"
	return;
	}
		
DOWNLOAD ()
	{
	# Get the proper file-type to set-up the Content-Type Parameter
	# We are relying on Linux's magic files here, not very trust-worthy, better systems are possible.
		OUTPUT=`cat ${TARGET}`
	CONTENT_TYPE=( `file -ib ${TARGET}` )
		MIME="${CONTENT_TYPE[0]}"
	# Get the file-size to ensure proper content-length headers
		CONTENT_LENGTH=( `wc -c ${TARGET}` )
#		CONTENT_LENGTH=( `echo "${OUTPUT}" | wc -c` )
	# Send the HTTP Headers
		HEADERS
	# Start the output for Download
		cat ${TARGET}
#		echo "${OUTPUT}"
	return;
	}

EXEC ()
	{
#	OUTPUT=`php -q ${BASE_DIR} 2>/dev/null`
	CONTENT_TYPE=( "text/html" )
	MIME="${CONTENT_TYPE[0]}"
#	CONTENT_LENGTH=( `echo "${OUTPUT}" | wc -c` )
		HEADERS
#	echo "${OUTPUT}"
	php -q ${TARGET}

	return;
	}
	
	
BROWSE_DIR ()
	{
#		CONTENT=`dir -1lgGp ${TARGET}`
#	CONTENT=`ls -lFHLgG ${TARGET}`
	CONTENT=`dir -gGlL --file-type ${TARGET}`
		HEADERS
		TOP
		SHOW_DIR
		BOTTOM
	return;
	}

SHOW_ERROR ()
	{
		echo "<table width=\"100%\" cellpadding=\"2\">"
		echo "<tr><td class=\"listhead\">Sorry Requested URL http://${WEB[0]}${HTTP_FILE} NOT Found!</td></tr>"
		echo "</table>"
	return;
	}
	
	
DISPLAY_ERROR ()
	{
		HEADERS
		TOP
		SHOW_ERROR
		BOTTOM
	return;
	}


	
MAIN ()
	{
	case ${ACTION} in
		BROWSE_DIR) BROWSE_DIR
		;;
		DOWNLOAD) DOWNLOAD
		;;
		EXECUTE) EXEC
		;;
		SHOW_ERROR) DISPLAY_ERROR
		;;
		*)
		;;
	esac
	
#	[ x"${DOWNLOAD}" == "xY" ] && DOWNLOAD || CREATE_PAGE
	return;
	}

MAIN
exit 0
